//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.DragDrop
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		private class CalendarDragData
		{
			public CalendarDragData(string eventType)
			{
				_eventType = eventType;
			}


			public string EventType
			{
				get { return _eventType; }
			}


			private string _eventType;
		}

		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			// Setup the dates displayed in the Schedule view
			DateTime today = DateTime.Now.Date;

			_schedule.TimetableSettings.Dates.Clear();
			for (int i = 0; i < 5; i++)
			{
				_schedule.TimetableSettings.Dates.Add(
					today + new TimeSpan(i * TimeSpan.TicksPerDay));
			}

			// Set the visible columns to 3
			_schedule.TimetableSettings.VisibleColumns = 3;

			// Note that we switch on the AllowDrop flag of the Calendar control.
			// This could be done directly from the property page of the
			// control, but we do it here for clarity
			_schedule.AllowDrop = true;

			// Mark schedule cells as custom drawn. This
			// would allow us to add visual indication to
			// the user, when he or she drags an item over the control
			_schedule.CustomDraw = CustomDrawElements.TimetableCell;
			_schedule.Draw += new EventHandler<DrawEventArgs>(this._schedule_Draw);

			_pointedDate = DateTime.MinValue;

			// Create different styles for different types of events
			_taskStyle = new Style();
			_taskStyle.TextColor = Color.FromArgb(0xFF, 0x80, 0x80, 0x80);
			_taskStyle.BorderTopColor = Color.DarkOrchid;
			_taskStyle.BorderLeftColor = Color.DarkOrchid;
			_taskStyle.BorderBottomColor = Color.DarkOrchid;
			_taskStyle.BorderRightColor = Color.DarkOrchid;
			_taskStyle.LineColor = Color.DarkOrchid;
			_taskStyle.FillColor = Color.MediumOrchid;
			_taskStyle.Brush = new MindFusion.Drawing.LinearGradientBrush(
				Color.White, Color.Plum, 90);
			_taskStyle.HeaderTextColor = Color.DarkViolet;

			_eventStyle= new Style();
			_eventStyle.TextColor = Color.FromArgb(0xFF, 0x80, 0x80, 0x80);
			_eventStyle.BorderTopColor = Color.Goldenrod;
			_eventStyle.BorderLeftColor = Color.Goldenrod;
			_eventStyle.BorderBottomColor = Color.Goldenrod;
			_eventStyle.BorderRightColor = Color.Goldenrod;
			_eventStyle.LineColor = Color.Goldenrod;
			_eventStyle.FillColor = Color.FromArgb(250, 200, 100);
			_eventStyle.Brush = new MindFusion.Drawing.LinearGradientBrush(
				Color.White, Color.PaleGoldenrod, 90);
			_eventStyle.HeaderTextColor = Color.DarkGoldenrod;

			_freeStyle = new Style();
			_freeStyle.TextColor = Color.FromArgb(0xFF, 0x80, 0x80, 0x80);
			_freeStyle.BorderTopColor = Color.LimeGreen;
			_freeStyle.BorderLeftColor = Color.LimeGreen;
			_freeStyle.BorderBottomColor = Color.LimeGreen;
			_freeStyle.BorderRightColor = Color.LimeGreen;
			_freeStyle.LineColor = Color.LimeGreen;
			_freeStyle.FillColor = Color.PaleGreen;
			_freeStyle.Brush = new MindFusion.Drawing.LinearGradientBrush(
				Color.White, Color.LightGreen, 90);
			_freeStyle.HeaderTextColor = Color.DarkGreen;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void _picAppointment_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// Start dragging an appointment
			DoDragDrop(new CalendarDragData("app"), DragDropEffects.Copy);
		}

		private void _picTask_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// Start dragging a task
			DoDragDrop(new CalendarDragData("task"), DragDropEffects.Copy);
		}

		private void _picEvent_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// Start dragging a general event
			DoDragDrop(new CalendarDragData("event"), DragDropEffects.Copy);
		}

		private void _picFree_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// Start dragging a free event
			DoDragDrop(new CalendarDragData("free"), DragDropEffects.Copy);
		}

		private void _schedule_DragOver(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if (!e.Data.GetDataPresent(typeof(CalendarDragData)))
			{
				e.Effect = DragDropEffects.None;
				return;
			}

			// Give a visual feedback of the user
			Point client = _schedule.PointToClient(new Point(e.X, e.Y));

			if (_rbMethod1.Checked)
			{
				_pointedDate = _schedule.GetDateAt(client);
			}

			else if (_rbMethod2.Checked)
			{
				_pointedDate = _schedule.GetExactDateAt(client);

				if (_pointedDate == DateTime.MinValue)
				{
					e.Effect = DragDropEffects.None;
					_pnlTip.Visible = false;
				}
				else
				{
					_lbTip.Text = _pointedDate.ToString("H:mm");
					_pnlTip.Visible = true;
				}
			}

			if (_pointedDate != DateTime.MinValue)
			{
				e.Effect = DragDropEffects.Copy;

				// Repaint
				_schedule.Invalidate();
			}
			else
			{
				e.Effect = DragDropEffects.None;
			}
		}

		private void _schedule_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if (!e.Data.GetDataPresent(typeof(CalendarDragData)))
				return;

			CalendarDragData data =
				e.Data.GetData(typeof(CalendarDragData)) as CalendarDragData;

			// Create the event
			switch (data.EventType)
			{

				case "app":
				{
					Appointment app = new Appointment();
					app.HeaderText = "Appointment";
					app.DescriptionText = "This is an appointment.";
					app.StartTime = _pointedDate;
					app.EndTime = _pointedDate +
						new TimeSpan(60 * TimeSpan.TicksPerMinute);
					_schedule.Schedule.Items.Add(app);
				}
					break;

				case "task":
				{
					// The task uses Appointment object with
					// different style
					Appointment task = new Appointment();
					task.HeaderText = "Task";
					task.DescriptionText = "This is a task.";
					task.StartTime = _pointedDate;
					task.EndTime = _pointedDate +
						new TimeSpan(60 * TimeSpan.TicksPerMinute);

					// Set different style for tasks
					task.Style = _taskStyle;

					_schedule.Schedule.Items.Add(task);
				}
					break;

				case "event":
				{
					// The general event uses Appointment object with
					// different style
					Appointment anEvent = new Appointment();
					anEvent.HeaderText = "Event";
					anEvent.DescriptionText = "This is a general event.";
					anEvent.StartTime = _pointedDate;
					anEvent.EndTime = _pointedDate +
						new TimeSpan(60 * TimeSpan.TicksPerMinute);

					// Set different style for tasks
					anEvent.Style = _eventStyle;

					_schedule.Schedule.Items.Add(anEvent);
				}
					break;

				case "free":
				{
					// The free event uses Appointment object with
					// different style
					Appointment free = new Appointment();
					free.HeaderText = "Leisure";
					free.DescriptionText = "Take a cup of coffee.";
					free.StartTime = _pointedDate;
					free.EndTime = _pointedDate +
						new TimeSpan(60 * TimeSpan.TicksPerMinute);

					// Set different style for tasks
					free.Style = _freeStyle;

					_schedule.Schedule.Items.Add(free);
				}
					break;

				default:
					return;

			}

			// No date is pointed
			_pointedDate = DateTime.MinValue;

			if (_rbMethod2.Checked)
				_pnlTip.Visible = false;

			// Repaint
			_schedule.Invalidate();
		}

		private void _schedule_DragLeave(object sender, System.EventArgs e)
		{
			// No date is pointed
			_pointedDate = DateTime.MinValue;

			// Repaint
			_schedule.Invalidate();
		}

		private void _schedule_Draw(object sender, DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.TimetableCell)
			{
				if (_rbMethod1.Checked)
				{
					if (_pointedDate != DateTime.MinValue)
					{
						TimeSpan segment = _schedule.TimetableSettings.CellTime;

						DateTime cellStart = e.Date + e.StartTime;
						DateTime cellEnd = e.Date + e.EndTime;

						DateTime prevCellStart = cellStart - segment;
						DateTime prevCellEnd = cellEnd - segment;

						Rectangle bounds = e.Bounds;
						bounds.Width -= 1;
						bounds.Height -= 1;

						// Fill the cell
						if (cellStart <= _pointedDate && _pointedDate < cellEnd)
						{
							// It is the first cell
							e.Graphics.FillRectangle(Brushes.LightSteelBlue, bounds);

							// Draw lines on the left, top and right
							bounds.Y += 1;
							bounds.Height -= 1;
							//e.Graphics.DrawLines(Pens.SteelBlue,
							//    new Point[]
							//{
							//    new Point(bounds.Left, bounds.Bottom),
							//    new Point(bounds.Left, bounds.Top),
							//    new Point(bounds.Right, bounds.Top),
							//    new Point(bounds.Right, bounds.Bottom),
							//});
							e.Graphics.DrawRectangle(Pens.SteelBlue, bounds);

							// Draw the time interval
							string interval = cellStart.ToString("hh:mm tt");
							e.Graphics.DrawString(interval, Font, Brushes.Black,
								bounds.Left + 2, bounds.Top + 2);
						}
						//if (prevCellStart <= _pointedDate && _pointedDate < prevCellEnd)
						//{
						//    // It is the second cell
						//    e.Graphics.FillRectangle(Brushes.LightSteelBlue, bounds);

						//    // Draw lines on the left, top and right
						//    bounds.Height -= 1;
						//    e.Graphics.DrawLines(Pens.SteelBlue,
						//        new Point[]
						//    {
						//        new Point(bounds.Left, bounds.Top),
						//        new Point(bounds.Left, bounds.Bottom),
						//        new Point(bounds.Right, bounds.Bottom),
						//        new Point(bounds.Right, bounds.Top)
						//    });

						//    // Draw an icon
						//    Image image = _hidden.Image;
						//    e.Graphics.DrawImage(image,
						//        bounds.X + bounds.Width - image.Width - 4,
						//        bounds.Y + bounds.Height / 2 - image.Height / 2);
						//}
					}
				}

				else if (_rbMethod2.Checked)
				{
					if (_pointedDate.Date == e.Date)
					{
						if (e.StartTime <= _pointedDate.TimeOfDay && _pointedDate.TimeOfDay <= e.EndTime)
						{
							// Draw a line, indicating the item's start time
							MindFusion.Scheduling.WinForms.TimetableSettings settings =
								_schedule.TimetableSettings;
							long off = _pointedDate.TimeOfDay.Ticks - e.StartTime.Ticks;
							int offPixels = (int)(((double)off / settings.CellTime.Ticks) * settings.CellSize);

							e.Graphics.DrawLine(Pens.Black, e.Bounds.Left + 1, e.Bounds.Y + offPixels,
								e.Bounds.Right - 1, e.Bounds.Y + offPixels);
						}
					}
				}
			}
		}

	
		private string _appPath;
		private DateTime _pointedDate;
		private Style _taskStyle;
		private Style _eventStyle;
		private Style _freeStyle;
	}
}
